import { scaleBand, scaleLinear, scaleSequential, scaleOrdinal, max, format, extent } from 'd3';
import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import ReactDropdown from 'react-dropdown';
import { config } from './config';
import { useData } from './useData';
import { getTop10BarData } from './sumData';
import { getLabel, attributes } from './getLabel';
import { AxisBottom } from './AxisBottom';
import { AxisLeft, AxisBarLeft } from './AxisLeft';
//import { getData } from './getData';
import { getUsAtlas } from './getUsAtlas';
import { Dropdown } from './Dropdown';
import { Marks } from './Marks';
import { BarMarks } from './BarMarks';
import { MapMarks } from './MapMarks';
import { CurveMarks } from './CurveMarks';

//Part I: Global values
const width = 960;
const menuHeight = 60;
const height = 500 - menuHeight;
const margin = { top: 10, right: 30, bottom: 65, left: 90 };
const xAxisLabelOffset = 30;
const yAxisLabelOffset = 50;
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;
const halfWidth = (innerWidth - 20) / 2;
const halfHeight = (innerHeight - 20) / 2;
const halfMarginX = 30;
const halfMarginY = 40;

//Part II: Main functiond
const App = () => {
  const data = useData();
  const usAtlas = getUsAtlas();

  //Step 0: Menu with attribute selections
  const initialXAttribute = 'Week';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);
  const xValue = d => d[xAttribute];
//  xLabel is fixed for X
//  const xAxisLabel = getLabel(xAttribute);
  const xAxisLabel = 'Week of the Year';

  const initialYAttribute = 'ILI_Percent';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = d => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);

  if (!data || !usAtlas) {
    return <pre>Loading...</pre>;
  }
  
  const siFormat = format('.2s');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');

  //Step 1: Derived data from here for plot:4-1 Line chart
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, halfWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([halfHeight, 0]);

  const colorValue = d => d.Year;
  const sizeValue = d => d.ILI_Total;
  const tooltipValue = d => d.ILI_Total;
  
  const sizeScale = scaleLinear()
    .domain(extent(data, sizeValue))
    .range([1.1, 8.8]);

  const colorScale = scaleOrdinal()
  	.domain(data.map(colorValue))
  	.range(['#2b8cbe','#fec44f','#de2d26','#31a354','#deebf7']);
//  	.range(['#2b8cbe','#fec44f','#de2d26','#756bb1','#deebf7','#31a354','#fff7bc']);
  
  //console.log(colorScale.domain());
  //console.log(colorScale.range());

  //Step 2: US map for all states plot: 4-2 
  console.log(usAtlas);
  const mapColorScale1 = scaleLinear()
  	.domain([0,60])
  	.range(["white", "blue"])
  const mapColorScale = scaleSequential()
   	.domain([0, 60])
   	.interpolator(d3.interpolatePuRd); 
//   	.interpolator(d3.interpolateViridis); //interpolatePuRd
  console.log(mapColorScale(50));
  
  //Step 3: Derived data from here for plot:4-3 Bar chart
  const top10BarData = getTop10BarData(data);

  const yBarValue = d => d.State;
  const xBarValue = d => d.Tvalue;
  
  const yBarScale = scaleBand()
    .domain(top10BarData.map(yBarValue))
    .range([0, halfHeight])
    .paddingInner(0.15);

  //This is for max, we should do sth different!
  const xBarScale = scaleLinear()
    .domain([0, max(top10BarData, xBarValue)])
    .range([0, halfWidth]);

  //Step 4: Derived line plot plot: 4-4 Curve chart 
//  const xAxisTickFormat = timeFormat('%a');
  //Aggregate data by weeks
  
  //Step 5: Draw plots using svg, other controls: ReactDropdown
  return (
    <>
      <div className="menus-container">
        <span className="dropdown-label">Choose a data field:</span>
        <ReactDropdown
          options={attributes}
          value={yAttribute}
          onChange={({ value }) => setYAttribute(value)}
        />
      </div>
      <svg width={width} height={height}>
        <g transform={`translate(${margin.left + halfWidth + halfMarginX},${margin.top}) scale(0.995 0.995)`}>
          <svg width={halfWidth} height={halfHeight+20} viewBox={`0 0 ${width} ${height+60}`} >
          	<MapMarks 
              atlas={usAtlas}         
              colorScale={mapColorScale}
            />
          </svg>
        </g>
        <g transform={`translate(${margin.left},${margin.top + halfHeight + halfMarginY})`}>
        <AxisBottom
          xScale={xBarScale}
          innerHeight={halfHeight}
          tickFormat={xAxisTickFormat}
        />
        <AxisBarLeft yScale={yBarScale} />
          <text
            className="axis-label"
            x={halfWidth / 2}
            y={halfHeight + xAxisLabelOffset}
            textAnchor="middle"
          >
          ILI Total
        </text>
        <BarMarks
          data={top10BarData}
          xScale={xBarScale}
          yScale={yBarScale}
          xValue={xBarValue}
          yValue={yBarValue}
          tooltipFormat={xAxisTickFormat}
        />
        </g>
        <g transform={`translate(${margin.left},${margin.top})`}>
          <AxisBottom
            xScale={xScale}
            innerHeight={halfHeight}
            tickFormat={xAxisTickFormat}
            tickOffset={5}
          />
          <text
            className="axis-label"
            textAnchor="middle"
            transform={`translate(${-yAxisLabelOffset},${halfHeight /
              2}) rotate(-90)`}
          >
            {yAxisLabel}
          </text>
          <AxisLeft yScale={yScale} innerWidth={halfWidth} tickOffset={5} />
          <text
            className="axis-label"
            x={halfWidth / 2}
            y={halfHeight + xAxisLabelOffset}
            textAnchor="middle"
          >
            {xAxisLabel}
          </text>
          <Marks
            data={data}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            clrScale={colorScale}
            clrValue={colorValue}
            sizeScale={sizeScale} 
            circleRadius={sizeValue}
            tooltipFormat={xAxisTickFormat}
            ttpValue={tooltipValue}
          />
        </g>
        <g transform={`translate(${margin.left + halfWidth + halfMarginX},${margin.top + halfHeight + halfMarginY})`}>
          <AxisBottom
            xScale={xScale}
            innerHeight={halfHeight}
            tickFormat={xAxisTickFormat}
            tickOffset={5}
          />
          <text
            className="axis-label"
            textAnchor="middle"
            transform={`translate(${-yAxisLabelOffset},${halfHeight /
              2}) rotate(-90)`}
          >
            {yAxisLabel}
          </text>
          <AxisLeft yScale={yScale} innerWidth={halfWidth} tickOffset={5} />
          <text
            className="axis-label"
            x={halfWidth / 2}
            y={halfHeight + xAxisLabelOffset}
            textAnchor="middle"
          >
            {xAxisLabel}
          </text>
          <Marks
            data={data}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            clrScale={colorScale}
            clrValue={colorValue}
            sizeScale={sizeScale} 
            circleRadius={sizeValue}
            tooltipFormat={xAxisTickFormat}
            ttpValue={tooltipValue}
          />
        </g>
      </svg>
    </>
  );
};



const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
